/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */
import { AttachmentBuilder, ChannelType, type Client } from "discord.js";
import { channels } from "../config/channels.js";
import { logger } from "../utils/logger.js";

const emptyArray = (array: Array<unknown>): void => {
  array.splice(0, array.length);
};

/**
 * Posts a daily standup reminder in configured channels.
 * @param client - The Discord client.
 * @param activeIds - The list of user IDs who sent a progress update yesterday.
 */
// eslint-disable-next-line max-lines-per-function -- shut up
export const standup = async(
  client: Client,
  activeIds: Array<string>,
): Promise<void> => {
  try {
    await client.users.
      send("465650873650118659", {
        content: `The following people sent a progress update yesterday:`,
        files:   [
          new AttachmentBuilder(Buffer.from(activeIds.join("\n")), {
            name: "ids.txt",
          }),
        ],
      }).
      catch(() => {
        void logger.log(
          "warn",
          "Failed to send progress update message to user.",
        );
      });
    emptyArray(activeIds);
    const mapped = await Promise.all(
      channels.map(async(channel) => {
        const fetched = await client.channels.
          fetch(channel.channelId).
          catch(() => {
            void logger.log("warn", `Failed to fetch channel ${channel.name}.`);
            return null;
          });
        if (!fetched) {
          await logger.log("warn", `Channel ${channel.name} not found.`);
          return null;
        }
        if (fetched.type !== ChannelType.GuildText) {
          await logger.log(
            "warn",
            `Channel ${channel.name} is not a text channel.`,
          );
          return null;
        }
        return fetched;
      }),
    );
    const filtered = mapped.filter((channel) => {
      return channel !== null;
    });
    await Promise.all(
      filtered.map(async(channel) => {
        const sent = await channel.
          send({ allowedMentions: {
            roles: [
              channels.find((c) => {
                return c.channelId === channel.id;
              })?.roleId ?? channel.guildId,
            ],
          },
          content:
            `Good morning <@&${
              channels.find((c) => {
                return c.channelId === channel.id;
              })?.roleId ?? channel.guildId
            }> It is time for your daily progress update. Please share the following in this thread:
          
1️⃣ What did you accomplish yesterday?
2️⃣ What are you working on today?
3️⃣ Are there any blockers or issues you need help with?` }).
          catch(() => {
            void logger.log(
              "warn",
              `Failed to send progress reminder in channel ${channel.name}.`,
            );
            return null;
          });
        if (sent) {
          await sent.
            startThread({
              autoArchiveDuration: 1440,
              name:                `Daily Progress Update - ${new Date().toLocaleDateString()}`,
            }).
            catch(() => {
              void logger.log(
                "warn",
                `Failed to start thread in channel ${channel.name}.`,
              );
              return null;
            });
        }
      }),
    );
  } catch (error) {
    if (error instanceof Error) {
      await logger.error("standup module", error);
      return;
    }
    await logger.error("standup module", new Error(String(error)));
  }
};
